# python 3 headers, required if submitting to Ansible
from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

DOCUMENTATION = """
        lookup: drupal_sites_from_drd
        author: Juergen Haas <juergen.haas@lakedrops.com>
        version_added: "0.9"
        short_description: get drupal sites from DRD (Drupal Remote Dashboard)
        description:
            - This lookup returns definition for the variable drupal_settings derived from a DRD API request.
        options:
          _hosts:
            description: Host name
            required: True
"""
from ansible.utils.display import Display
from ansible.plugins.lookup import LookupBase
from io import BytesIO
from six import string_types
import json
import pycurl

display = Display()

class LookupModule(LookupBase):

    def run(self, hosts, variables=None, drd_url = '', drd_username = '', drd_password = '', **kwargs):
        ret = []

        display.verbose('Logging into %s' % drd_url)
        c = pycurl.Curl()
        c.setopt(c.URL, '%s/user/login?_format=json' % drd_url)
        c.setopt(c.POST, 1)
        c.setopt(c.HTTPHEADER, ['Content-Type: application/json'])
        c.setopt(pycurl.POSTFIELDS, json.dumps({
            'name': drd_username,
            'pass': drd_password,
        }))
        c.setopt(c.COOKIEFILE, '')
        c.perform()
        c.setopt(c.POST, 0)
        if c.getinfo(c.RESPONSE_CODE) == 200:
            for host in hosts:
                display.verbose('Reading Drupal sites for host %s' % host)
                buffer = BytesIO()
                c.setopt(c.URL, '%s/jsonapi/drd_host/drd_host?filter[name]=%s' % (drd_url, host))
                c.setopt(c.WRITEDATA, buffer)
                c.perform()
                for drdHost in json.loads(buffer.getvalue().decode('utf-8'))['data']:
                    display.verbose('Reading cores for host %s' % drdHost['attributes']['name'])
                    buffer = BytesIO()
                    c.setopt(c.URL, '%s/jsonapi/drd_core/drd_core?filter[host.id]=%s' % (drd_url, drdHost['id']))
                    c.setopt(c.WRITEDATA, buffer)
                    c.perform()
                    for drdCore in json.loads(buffer.getvalue().decode('utf-8'))['data']:
                        # noinspection PyPep8Naming
                        drdCoreAttributes = drdCore['attributes']
                        if drdCoreAttributes['status']:
                            if drdCoreAttributes['drupalroot'] is None or drdCoreAttributes['drupalroot'] == '':
                                drdCoreAttributes['drupalroot'] = '/var/www/drupal%s' % drdCoreAttributes['drupal_internal__id']
                            if drdCoreAttributes['drupalroot'][:9] == '/var/www/':
                                values = {
                                    'coreid': drdCoreAttributes['drupal_internal__id'],
                                    'webroot': drdCoreAttributes['drupalroot'][8:],
                                    'repo': drdCoreAttributes['gitrepo'],
                                    'branch': 'main',
                                    'domain': '',
                                    'project': drdCoreAttributes['name'],
                                }

                                display.verbose('Reading domains for core %s' % drdCoreAttributes['name'])
                                buffer = BytesIO()
                                c.setopt(c.URL, '%s/jsonapi/drd_domain/drd_domain?filter[core.id]=%s' % (drd_url, drdCore['id']))
                                c.setopt(c.WRITEDATA, buffer)
                                c.perform()
                                for drdDomain in json.loads(buffer.getvalue().decode('utf-8'))['data']:
                                    # noinspection PyPep8Naming
                                    drdDomainAttributes = drdDomain['attributes']
                                    if drdDomainAttributes['status']:
                                        display.verbose('Preparing domain %s (%s)' % (drdDomainAttributes['domain'], drdDomainAttributes['name']))
                                        values.__setitem__('domain', drdDomainAttributes['domain'])
                                if values['domain'] != '':
                                    ret.append(self.expand(dict(variables.get('drupal_settings_template')), values))

        c.close()
        if len(ret) > 1:
            return ret
        return [ret]

    def expand(self, itemplate, values):
        template = dict(itemplate)
        for key in template:
            if isinstance(template.get(key), string_types):
                template.__setitem__(key, self.replace(template.get(key), values))
            elif isinstance(template.get(key), dict):
                template.__setitem__(key, self.expand(template.get(key), values))
            elif isinstance(template.get(key), list):
                expanded = []
                for item in template.get(key):
                    if isinstance(item, string_types):
                        expanded.append(self.replace(item, values))
                    elif isinstance(item, dict):
                        expanded.append(self.expand(item, values))
                template.__setitem__(key, expanded)
        return template

    @staticmethod
    def replace(string, values):
        for key in values:
            string = string.replace('___%s___' % key, str(values.get(key)))
        return string
